APPLICATION_EVENT_PUBLISHER = '__application_event_publisher'
CONFIGURATION_ANNOTATION_PROCESSOR = '__configuration_annotation_processor'
CONFIGURATION_BEAN_NAME_GENERATOR = '__configuration_bean_name_generator'
AUTOWIRED_ANNOTATION_PROCESSOR = '__autowired_annotation_processor'
